import json
import asyncio
import signal
from loguru import logger

from bots.matrix import MatrixBot
from rmq.consumers import RMQConsumer


def load_config(path: str) -> dict:
    with open(path, 'r', encoding='utf-8') as f:
        return json.loads(f.read())


async def main():
    # Load configuration
    config = load_config('config.json')

    # Initialize Matrix bot
    bot = MatrixBot(config['matrix'])

    # Initialize RabbitMQ consumer
    consumer = RMQConsumer(
        host='localhost',
        exchange='matrix_alerts'
    )

    # RabbitMQ message handler
    async def on_message(ch, method, properties, body):
        try:
            logger.debug(f"Event received: {body}")
            message = json.loads(body)
            await bot.send_markdown(message['body'])
            logger.debug(f"Message sent to Matrix: {message['body']}")
        except Exception as e:
            logger.error(f"Failed to send message to Matrix: {e}")

    # Ensure cleanup on shutdown
    def shutdown():
        logger.info("Shutdown initiated.")
        all_tasks = asyncio.all_tasks(loop)
        for task in all_tasks:
            task.cancel()

    loop = asyncio.get_event_loop()
    loop.add_signal_handler(signal.SIGINT, shutdown)
    loop.add_signal_handler(signal.SIGTERM, shutdown)

    # Start the RabbitMQ consumer
    consume_task = asyncio.create_task(consumer.start_consuming(on_message))

    try:
        # Await the consume task to keep the main running until shutdown
        await consume_task
    except asyncio.CancelledError:
        logger.info("Main task cancellation requested.")
    finally:
        logger.info("Stopping consumer and cleaning up resources.")

        # Stop the RabbitMQ consumer if it is running
        if consumer.channel:
            try:
                await consumer.stop_consuming()
            except Exception as e:
                logger.warning(f"Failed to stop consumer cleanly: {e}")

        # Close the Matrix bot
        logger.info("Shutting down bot.")
        await bot.close()

        logger.info("Shutdown complete.")


def main_with_shutdown():
    loop = asyncio.get_event_loop()
    main_task = loop.create_task(main())

    try:
        loop.run_until_complete(main_task)
    except asyncio.CancelledError:
        logger.info("Main task has been cancelled.")
    finally:
        pending_tasks = [t for t in asyncio.all_tasks(loop) if not t.done()]
        if pending_tasks:
            loop.run_until_complete(asyncio.gather(*pending_tasks, return_exceptions=True))
        loop.run_until_complete(loop.shutdown_asyncgens())
        loop.close()
        logger.info("Event loop closed.")


if __name__ == '__main__':
    main_with_shutdown()
