import asyncio
import pika
from loguru import logger
from pika.channel import Channel
from typing import Callable, Optional, Awaitable


class RMQConsumer:
    """
    A handler class for RabbitMQ to manage connection and message consumption.

    Attributes:
        host (str): RabbitMQ server host.
        queue_name (str): Name of the queue.
        exchange (str): Name of the exchange.
        exchange_type (str): Type of the exchange (default is 'fanout').
        port (int): Port number for RabbitMQ server (default is 5672).
        virtual_host (str): Virtual host (default is '/').
        username (str): Username for RabbitMQ (default is 'guest').
        password (str): Password for RabbitMQ (default is 'guest').
    """

    def __init__(self, host: str, queue_name: str = '', exchange: str = '', exchange_type: str = 'fanout',
                 port: int = 5672, virtual_host: str = '/', username: str = 'guest', password: str = 'guest'):
        self.host = host
        self.queue_name = queue_name
        self.exchange = exchange
        self.exchange_type = exchange_type
        self.port = port
        self.virtual_host = virtual_host
        self.username = username
        self.password = password
        self.connection: Optional[pika.BlockingConnection] = None
        self.channel: Optional[Channel] = None

    def connect(self) -> None:
        """Establish a connection to RabbitMQ and set up the channel and queue."""
        credentials = pika.PlainCredentials(self.username, self.password)
        parameters = pika.ConnectionParameters(
            host=self.host,
            port=self.port,
            virtual_host=self.virtual_host,
            credentials=credentials
        )

        try:
            self.connection = pika.BlockingConnection(parameters)
            self.channel = self.connection.channel()
            self.channel.exchange_declare(exchange=self.exchange, exchange_type=self.exchange_type)

            result = self.channel.queue_declare(queue=self.queue_name, durable=True)
            self.queue_name = result.method.queue
            self.channel.queue_bind(exchange=self.exchange, queue=self.queue_name)
        except pika.exceptions.AMQPError as error:
            logger.error(f"Error connecting to RabbitMQ: {error}")

    async def start_consuming(self, on_message_callback: Callable[
        [Channel, pika.spec.Basic.Deliver, pika.spec.BasicProperties, bytes], Awaitable[None]]
    ) -> None:
        """
        Start consuming messages from the queue.

        Args:
            on_message_callback (Callable): User-defined callback function for processing messages.
        """

        def callback_wrapper(ch, method, properties, body):
            asyncio.create_task(on_message_callback(ch, method, properties, body))

        self.connect()
        if not self.channel:
            logger.error("Failed to connect to RabbitMQ.")
            return

        self.channel.basic_consume(queue=self.queue_name, on_message_callback=callback_wrapper, auto_ack=True)
        await self._consume_async()

    async def stop_consuming(self) -> None:
        """Stop consuming messages and close the connection."""
        if self.channel and self.connection:
            self.channel.stop_consuming()
            await self.close()

    async def _consume_async(self) -> None:
        """Asynchronously process events."""
        while self.channel and self.channel.is_open:
            self.connection.process_data_events(time_limit=1)
            await asyncio.sleep(0.1)

    async def close(self) -> None:
        """Close the RabbitMQ connection."""
        if self.connection and not self.connection.is_closed:
            self.connection.close()
